# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Dict
from vpc_img_inst.config_builder import ConfigBuilder
from vpc_img_inst.utils import color_msg, Color, logger, store_output, append_random_suffix
from ibm_cloud_sdk_core import ApiException

class VSIConfig(ConfigBuilder):

    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)

    def run(self) -> Dict[str, Any]:
        selected_vsi = self.create_new_instance()

        self.base_config['node_config']['vsi_id'] = selected_vsi['id']
        store_output({'vsi_id':selected_vsi['id']},self.base_config)
        return self.base_config
    
    def create_new_instance(self):
        security_group_identity_model = {"id": self.base_config["node_config"]["security_group_id"]}
        subnet_identity_model = {"id": self.base_config["node_config"]["subnet_id"]}
        primary_network_interface = {
            "name": "eth0",
            "subnet": subnet_identity_model,
            "security_groups": [security_group_identity_model],
        }
        vsi_name = append_random_suffix(base_name="temp-vsi")
        storage_volume_name = append_random_suffix(base_name=vsi_name+"-boot") 


        boot_volume_profile = {
            "capacity": self.base_config["node_config"]["boot_volume_capacity"] if self.base_config["node_config"]["boot_volume_capacity"] else 100,
            "name": storage_volume_name,
            "profile": {
                "name": self.base_config["node_config"].get("volume_tier_name", "general-purpose")
            },
        }

        boot_volume_attachment = {
            "delete_volume_on_instance_delete": True,
            "volume": boot_volume_profile,
        }

        key_identity_model = {"id": self.base_config["node_config"]["key_id"]}
        profile_name = self.base_config["node_config"]["instance_profile_name"]
        profile_name = "bx2-2x8" if not profile_name else profile_name

        instance_prototype = {}
        instance_prototype["name"] = vsi_name
        instance_prototype["keys"] = [key_identity_model]
        instance_prototype["profile"] = {"name": profile_name}
        instance_prototype["resource_group"] = {"id": self.base_config["node_config"]["resource_group_id"]}
        instance_prototype["vpc"] = {"id": self.base_config["node_config"]["vpc_id"]}
        instance_prototype["image"] = {"id": self.base_config["node_config"]["image_id"]}

        instance_prototype["zone"] = {"name": self.base_config["zone_name"]}
        instance_prototype["boot_volume_attachment"] = boot_volume_attachment
        instance_prototype["primary_network_interface"] = primary_network_interface

        try:
            resp = self.ibm_vpc_client.create_instance(instance_prototype).result
        except ApiException as e:
            if e.code == 400 and "already exists" in e.message:
                logger.error("VSI already exists")
            elif e.code == 400 and "over quota" in e.message:
                logger.error("Create VM instance {} failed due to quota limit")
            else:
                logger.error(
                    "Create VM instance failed with status code {}".format(
                         str(e.code)
                    )
                )
            raise e

        logger.info(color_msg(f"Created VM instance: {vsi_name} with id: {resp['id']}",color=Color.LIGHTGREEN))
        return resp



# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import click
import pkg_resources
import yaml
from vpc_img_inst import config_modules
from vpc_img_inst.config_modules.delete_resources import clean_up
from vpc_img_inst.constants import DEFAULTS

from vpc_img_inst.utils import (Color, color_msg, create_folders,
                                    get_confirmation, logger, verify_paths,
                                    get_installation_types_for_feature,get_supported_features)

@click.command()
@click.option('--output-folder', '-o', show_default=True, default=DEFAULTS['output_folder'], help='Path to folder storing IDs of resources created by this program and installation logs')
@click.option('--iam-api-key', '-a', help='IAM_API_KEY')
@click.option('--version', '-v', help=f'Get package version', is_flag=True)
@click.option('--region','-r', show_default=True, default=DEFAULTS['region'], help='IBM Cloud region, e.g: us-south, us-east, eu-de, eu-gb.')
@click.option('--yes','-y',show_default=True, is_flag=True, help='Skips confirmation requests')
@click.option('--base-image-name','-im', show_default=True, default=DEFAULTS['base_image_name'], help='Prefix of image names from your account, on which the produced image will be based')
@click.option('--installation-type','-it',show_default=True, default=DEFAULTS['installation_type'], help='type of installation to use, e.g. for feature "cuda" the currently supported types are: "ubuntu" and "rhel".')
@click.option('--feature','-f', show_default=True,default=[DEFAULTS['feature']], multiple=True, help='Feature to install on the produced image. Currently: "cuda" or "docker".')
@click.option('--cleanup','-c', show_default=True, default=None, help='Path to a resources file, that will be submitted for deletion. Program will be terminated subsequently.')
@click.option('--compute-iam-endpoint', help='IAM endpoint url used for compute instead of default https://iam.cloud.ibm.com')
def builder(iam_api_key, output_folder, version, region, yes, base_image_name, installation_type, feature, cleanup, compute_iam_endpoint):
    if cleanup:
        clean_up(cleanup)
        exit(0)

    if version:
        print(f"{pkg_resources.get_distribution('vpc-img-inst').project_name}"
              f"-{pkg_resources.get_distribution('vpc-img-inst').version}")
        exit(0)

    logger.info((color_msg("VPC Image Installer Starting...", color=Color.YELLOW)))

    create_folders() # creates user scripts folder if does not exist

    # if input_file is empty, path to defaults.py is returned.
    input_file, output_file = verify_paths("", output_folder)
    
    with open(input_file) as f:
        base_config = yaml.safe_load(f)
    
    base_config['output_folder'] = output_file[:output_file.rfind(os.sep)+1]
    base_config['output_file'] = output_file

    base_config, modules = validate_api_keys(base_config, iam_api_key, compute_iam_endpoint)
    base_config['yes'] = yes

    base_config['region'] =  region.lower() 
    base_config['installation_type'] = installation_type
    base_config['user_image_name'] = base_image_name
    base_config['feature'] = feature
    
    logger.info(color_msg(f"""\n\nBase Image Name: {base_config['user_image_name']}\nFeature: {','.join(feature)}\nInstallation Type: {base_config['installation_type']}\nRegion: {base_config['region']}\nImage Creation Retries: {DEFAULTS['image_create_retries']} """, color=Color.YELLOW))
    logger.info(color_msg(f"\n\nCreated resources will be logged in {output_file}.\n", color=Color.YELLOW))
    if not yes:
        confirmation = get_confirmation(f"Proceed?")['answer']
        if not confirmation:
            sys.exit(0)

    base_config['scripts_path']=get_scripts_path(base_config['feature'],base_config['installation_type'])

    for module in modules:
        next_module = module(base_config)
        try:
            base_config = next_module.run()
        except Exception as e:
            logger.critical(f"Exception in {next_module.__class__.__name__} \n{e}")
            logger.info(color_msg("Program failed. Deleting created resources",color=Color.RED))
            clean_up(output_file)
            sys.exit(1)
        
    
    logger.info(color_msg(f"\n\nProgram Concluded.\nCreated resources are logged in {output_file}.\n", color=Color.YELLOW))

def validate_api_keys(base_config, iam_api_key, compute_iam_endpoint):
    """
    validates the api key specified.
    returns a base config dict, updated with the api-key field populated and the post api-key module removal modules list
    """
    # The API_KEY module is invoked and popped from the list. 
    modules = config_modules.MODULES
    api_key_module = modules[0]
    base_config = api_key_module(base_config).run(api_key=iam_api_key,
                                                  compute_iam_endpoint=compute_iam_endpoint)

    modules = modules[1:]
    return base_config , modules

def get_scripts_path(features,installation_type):
    """returns script path to execute in case both the specified feature exists and the installation type exits for it"""
    scripts = []
    supported_features = get_supported_features()
    for feature in features:
        
        if feature not in supported_features:
            logger.critical(color_msg(f"Feature Chosen: {feature} isn't supported ",color=Color.RED))
            raise Exception("Invalid feature argument.")
        
        inst_types = get_installation_types_for_feature(feature)
        if installation_type not in inst_types:
            logger.critical(color_msg(f"Installation type Chosen: {installation_type} isn't supported for feature: {feature}",color=Color.RED))
            raise Exception("Invalid installation type argument.")
        scripts.append(inst_types[installation_type])
        
    return scripts 

if __name__ == "__main__":
    builder()


# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Dict
from vpc_img_inst.config_builder import ConfigBuilder, spinner
from vpc_img_inst.utils import color_msg, Color, logger, store_output, append_random_suffix
from vpc_img_inst.constants import DEFAULTS
import time


class ImageCreate(ConfigBuilder):

    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)
        self.retries = DEFAULTS['image_create_retries']
        self.features = self.base_config['feature']

    def run(self) -> Dict[str, Any]:
        instance_volume_attachments = self.ibm_vpc_client.list_instance_volume_attachments(self.base_config['node_config']['vsi_id']).get_result()
        boot_volume_id = instance_volume_attachments['volume_attachments'][0]['volume']['id']
        default_image_name = ''
        for feature in self.features:
            default_image_name += feature+'-'
        default_image_name += f"""{self.base_config['node_config']['image_name']}"""  
        image_name = append_random_suffix(base_name=default_image_name)
        
        logger.info("Stopping VM instance...")
        self.ibm_vpc_client.create_instance_action(self.base_config['node_config']['vsi_id'], "stop")
        # blocks until instance stopped.
        self.poll_instance_status(self.base_config['node_config']['vsi_id']) 
        
        if self.retries:
            self.retries -= 1
            self.create_image(image_name, self.base_config['node_config']['resource_group_id'], boot_volume_id)
            return self.base_config
    
    
    def create_image(self, image_name, resource_group_id, boot_volume_id):
        payload = {"name": image_name,
                    "resource_group":{"id":resource_group_id},
                    "source_volume": {"id": boot_volume_id}}    
        image_data = self.ibm_vpc_client.create_image(payload).result
        logger.info(color_msg(f"""Creating image: "{image_name}" with id: "{image_data['id']}". Process may take a while. """, Color.YELLOW))
  
        self.base_config['new_image_id'] = image_data['id']
        # blocking call that forces the program to wait for the creation of the image.  
        response = self.poll_image_status(image_data['id'],image_name)

    @spinner
    def poll_image_status(self,image_id, image_name):
        tries = 300 # waits up to 50 min with 10 sec intervals
        sleep_interval = 10
        msg = ""
        while tries:
            image_status = self.ibm_vpc_client.get_image(image_id).result['status']
            if image_status == 'available':
                logger.info(color_msg(f"""\nImage named: "{image_name}" with id: "{image_id}" was created successfully.\n""", Color.LIGHTGREEN))
                store_output({"new_image":{'image_id':image_id, 'image_name':image_name}},self.base_config)
                return True
            elif image_status in ['failed','unusable','deprecated','deleting']:
                logger.info(color_msg(f"failed to create image {image_id}. Retrying...", Color.RED))
                store_output({f"failed_image-{DEFAULTS['image_create_retries']-self.retries}":{'image_id':image_id, 'image_name':image_name}},self.base_config)
                time.sleep(10)
                self.run()  # retry
                return True
            else:
                tries -= 1
                time.sleep(sleep_interval)
        logger.critical(color_msg(f"Failed to create image {image_id} within the expected time frame of {tries*sleep_interval/60} minutes.\n", Color.RED))
        return False

    @spinner
    def poll_instance_status(self, instance_id):
        tries = 30 # waits up to 1 min with 2 sec intervals
        sleep_interval = 2
        msg = ""
        while tries:
            instance_status = self.ibm_vpc_client.get_instance(instance_id).result['status']
            if instance_status == 'stopped':
                return True
            elif instance_status in ['failed']:
                logger.info(color_msg("\nInstance failed to stop.\n",color=Color.RED))
                return False
            else:
                tries -= 1
                time.sleep(sleep_interval)
        logger.critical(color_msg(f"\nInstance failed to stop within expected time frame of {tries*sleep_interval/60} minutes.\n",color=Color.RED))
        return False



# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import time
import yaml
import sys
from typing import Any, Dict

from vpc_img_inst.constants import DEFAULTS
sys.path.append('../generate_gpu_image')
from vpc_img_inst.config_builder import ConfigBuilder
from vpc_img_inst.utils import color_msg, Color, logger
from ibm_cloud_sdk_core import ApiException

class DeleteResources(ConfigBuilder):
    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)
        self.base_config = base_config
        if 'delete_resources' in base_config:
            self.ibm_vpc_client.set_service_url(self.base_config['endpoint'] + '/v1')
            self.delete_config()

    def run(self) -> Dict[str, Any]:
        self.delete_config()
        return self.base_config

    def delete_config(self):
        print("---------------------------------------------")

        if 'ip_id' in self.base_config['node_config'] and self.base_config['node_config']['ip_id']:
            try:
                self.ibm_vpc_client.delete_floating_ip(self.base_config['node_config']['ip_id'])
                logger.info(color_msg(f"""Deleted ip id: {self.base_config['node_config']['ip_id']} of address: {self.base_config['node_config']['ip_address']}""", color=Color.PURPLE))
            except ApiException as e:
                if e.code == 404:  # resource was already deleted
                    logger.info(f"""Floating ip with id: "{self.base_config['node_config']['ip_id']}" of address "{self.base_config['node_config']['ip_address']}" doesn't exist""")
                else:
                    raise e

        if 'vsi_id' in self.base_config['node_config'] and self.base_config['node_config']['vsi_id']:    
            # instance_data = self.ibm_vpc_client.get_instance(self.base_config['node_config']['vsi_id']).get_result()
        # delete instance
            try:
                self.ibm_vpc_client.delete_instance(self.base_config['node_config']['vsi_id'])  
                # blocking call waiting for instance to delete. 
                res=self.poll_instance_exists(self.base_config['node_config']['vsi_id']) 
            except ApiException as e:
                if e.code == 404:  # resource was already deleted
                    logger.info(f"""VSI with id: "{self.base_config['node_config']['vsi_id']}" doesn't exist""")
                else:
                    raise e            
        

        # delete subnet
        if 'subnet_id' in self.base_config["node_config"] and self.base_config["node_config"]['subnet_id']:
            try:
                self.ibm_vpc_client.delete_subnet(self.base_config["node_config"]['subnet_id'])
                # blocking call waiting for subnet to delete.
                self.poll_subnet_exists(self.base_config["node_config"]['subnet_id'])
            except ApiException as e:
                if e.code == 404:  # resource was already deleted
                    logger.info(f"""Subnet with id: "{self.base_config["node_config"]['subnet_id']}" doesn't exist""")
                else:
                    raise e
            
        # delete ssh key
        if 'key_id' in self.base_config["node_config"] and self.base_config["node_config"]['key_id']:
            try:
                self.ibm_vpc_client.delete_key(id=self.base_config["node_config"]['key_id'])
                logger.info(color_msg(f'Removed SSH key id: {self.base_config["node_config"]["key_id"]} from resource group', color=Color.PURPLE))
            except ApiException as e:
                if e.code == 404:
                    logger.info(f"""SSH key with id: "{self.base_config["node_config"]["key_id"]}" doesn't exist""")
                else:
                    raise e
            
        if 'ssh_private_key' in self.base_config['auth'] and self.base_config['auth']['ssh_private_key']:
            private_key = os.path.expanduser(self.base_config['auth']['ssh_private_key'])
            try:
                os.remove(private_key)
                logger.info(color_msg(f'Deleted private key: {private_key}', color=Color.PURPLE))
            except OSError as e:
                logger.info(f"""Local private key: {private_key} doesn't exist""")
            
            public_key = private_key+".pub"
            try:
                os.remove(public_key)
                logger.info(color_msg(f'Deleted public key: {public_key}', color=Color.PURPLE))
            except OSError as e:
                logger.info(f"""Local public key: {public_key} doesn't exist""")
        
        # delete vpc
        if 'vpc_id' in self.base_config["node_config"] and self.base_config["node_config"]['vpc_id']:
            try:
                self.ibm_vpc_client.delete_vpc(self.base_config["node_config"]['vpc_id'])
                logger.info(color_msg('Deleted VPC with id: {}'.format(self.base_config["node_config"]['vpc_id']), color=Color.PURPLE))
            except ApiException as e:
                if e.code == 404:
                    logger.info(f"""VPC with id: "{self.base_config["node_config"]['vpc_id']}" doesn't exist""")
                else:
                    raise e

        # delete failed images
        failed_images = _get_failed_images(self.base_config['output_file'])
        for failed_image_id in failed_images:
            try:
                self.ibm_vpc_client.delete_image(failed_image_id)
                logger.info(color_msg('Deleted failed image with id: {}'.format(failed_image_id), color=Color.PURPLE))
            except ApiException as e:
                if e.code == 404:  # resource was already deleted
                    pass
                else:
                    raise e            


    def poll_instance_exists(self,instance_id):
        tries = 30 # waits up to 5 min with 10 sec interval
        sleep_interval = 10
        msg = ""
        while tries:
            try:
                instance_data = self.ibm_vpc_client.get_instance(instance_id).result
            except Exception:
                logger.info(color_msg('Deleted VM instance with id: {}'.format(instance_id), color=Color.PURPLE))
                return True
            
            #print("Retrying... current status:", instance_data['status'])
            tries -= 1
            time.sleep(sleep_interval)
        logger.info(f"\Failed to delete instance within expected time frame of {tries*sleep_interval/60} minutes.\n")
        return False

    def poll_subnet_exists(self,subnet_id):
        tries = 30 # waits up to 5 min with 10 sec interval
        sleep_interval = 10
        msg = ""
        while tries:
            try:
                subnet_data = self.ibm_vpc_client.get_subnet(subnet_id).result
            except Exception:
                logger.info(color_msg('Deleted subnet id: {}'.format(self.base_config["node_config"]['subnet_id']), color=Color.PURPLE))
                return True
            
            #print("Retrying... current status:", subnet_data['status'])
            tries -= 1
            time.sleep(sleep_interval)
        logger.info(f"\Failed to delete instance within expected time frame of {tries*sleep_interval/60} minutes.\n")
        return False


def clean_up(output_file=None):
    base_config = {'delete_resources':True}
    if output_file and not os.path.exists(os.path.abspath(os.path.expanduser(output_file))):
        logger.critical(color_msg(f"file {output_file} doesn't exist in path",color=Color.RED))
        sys.exit(1)
    file = output_file if output_file else f"{DEFAULTS['output_folder']}created_resources"
    base_config['output_file'] = file
    with open(file, 'r') as f:
        resources = yaml.safe_load(f)
    
    node_config={"node_config":{"subnet_id":resources['subnet_id'] if 'subnet_id' in resources else '',
                                 "vpc_id":resources['vpc_id'] if 'vpc_id' in resources else '',
                                 "vsi_id":resources['vsi_id'] if 'vsi_id' in resources else '',
                                 "key_id":resources['key_id'] if 'key_id' in resources else '',
                                  "ip_id": resources['ip_id'] if 'ip_id' in resources else '' ,
                                  "ip_address": resources['ip_address'] if 'ip_address' in resources else '' ,}
                }
    auth = {"auth":{"ssh_private_key":resources['ssh_private_key'] if 'ssh_private_key' in resources else ''}}

    base_config.update({"iam_api_key": resources['iam_api_key']})
    base_config.update({"endpoint": resources['endpoint']})
    base_config.update(auth)
    base_config.update(node_config)
    # __init__ of DeleteResources starts the clean-up process when this module is run independently.     
    DeleteResources(base_config)  

def _get_failed_images(file):
    failed_images = []

    if not os.path.isfile(file):
        logger.error(color_msg("Failed to fetch failed images. Resources file doesn't exist",color=Color.RED))
        return failed_images
    else:
        with open(file, 'r') as f:
            resources = yaml.safe_load(f)
    
    for resource_key in resources:
        if 'failed_image' in resource_key:  # failed images are keys in the format: failed_image_<NUMBER>
            failed_images.append(resources[resource_key]['image_id'])
    return failed_images


if __name__ == "__main__":
    resources_file = None
    if len(sys.argv)>1:
        resources_file = sys.argv[1]
    clean_up(resources_file)


# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Dict
from vpc_img_inst.config_builder import ConfigBuilder, spinner
from vpc_img_inst.utils import find_default,get_option_from_list,get_region_by_endpoint, CACHE, logger, append_random_suffix, store_output, color_msg, Color

class VPCConfig(ConfigBuilder):

    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)
        self.base_config = base_config

        self.sg_rules = {'outbound_tcp_all': 'selected security group is missing rule permitting outbound TCP access\n',
                        'inbound_tcp_22': 'selected security group is missing rule permitting inbound traffic to tcp port 22 required for ssh\n',}  # security group rules.
        self.vpc_name = 'temp-vpc'
        
    def _get_region(self):
        region = None
        try:
            region = get_region_by_endpoint(self.ibm_vpc_client.service_url)
        except Exception:
            region = ConfigBuilder.region
        return region

    def run(self) -> Dict[str, Any]:
        region = self._get_region()

        vpc_obj, zone_obj = self._select_vpc(self.base_config, region)

        if not vpc_obj:
            raise Exception(f'Failed to select VPC')

        all_subnet_objects = self.ibm_vpc_client.list_subnets().get_result()[
            'subnets']

        # filter only subnets from selected availability zone
        subnet_objects = [s_obj for s_obj in all_subnet_objects if s_obj['zone']
                          ['name'] == zone_obj['name'] and s_obj['vpc']['id'] == vpc_obj['id']]

        if not subnet_objects:
            raise f'Failed to find subnet for vpc {vpc_obj["name"]} in zone {zone_obj["name"]}'
        
        self.base_config['node_config'].update({'vpc_id':vpc_obj['id'],
                                                'resource_group_id':CACHE['resource_group_id'],
                                                'security_group_id':vpc_obj['default_security_group']['id'],
                                                'subnet_id':subnet_objects[0]['id']})

        self.base_config['zone_name'] = zone_obj['name']                                                
        return self.base_config

    def _build_security_group_rule_prototype_model(self, missing_rule, sg_id=None):
        direction, protocol, port = missing_rule.split('_')
        remote = {"cidr_block": "0.0.0.0/0"}

        try: # port number was specified
            port = int(port)
            port_min = port
            port_max = port
        except:
            port_min = 1
            port_max = 65535

            # only valid if security group already exists
            if port == 'sg':
                if not sg_id:
                    return None
                remote = {'id': sg_id}

        return {
            'direction': direction,
            'ip_version': 'ipv4',
            'protocol': protocol,
            'remote': remote,
            'port_min': port_min,
            'port_max': port_max
        }

    def _create_vpc_peripherals(self, ibm_vpc_client, vpc_obj, zone_obj, resource_group):
        vpc_name = vpc_obj['name']
        vpc_id = vpc_obj['id']
        # create subnet
        subnet_name = '{}-subnet'.format(vpc_name)
        subnet_data = None

        # find cidr
        ipv4_cidr_block = None
        res = ibm_vpc_client.list_vpc_address_prefixes(
            vpc_id).result
        address_prefixes = res['address_prefixes']

        # searching for the CIDR block (internal ip range) matching the specified zone of a VPC (whose region has already been set) 
        for address_prefix in address_prefixes:
            if address_prefix['zone']['name'] == zone_obj['name']:
                ipv4_cidr_block = address_prefix['cidr']
                break

        subnet_prototype = {}
        subnet_prototype['zone'] = {'name': zone_obj['name']}
        subnet_prototype['ip_version'] = 'ipv4'
        subnet_prototype['name'] = subnet_name
        subnet_prototype['resource_group'] = resource_group
        subnet_prototype['vpc'] = {'id': vpc_id}
        subnet_prototype['ipv4_cidr_block'] = ipv4_cidr_block

        subnet_data = ibm_vpc_client.create_subnet(
            subnet_prototype).result
        subnet_id = subnet_data['id']
        store_output({"subnet_id":subnet_id},self.base_config)
        logger.info(color_msg(f"Created subnet: {subnet_prototype['name']} with id: {subnet_id}",color=Color.LIGHTGREEN))

        # Update security group to have all required rules
        sg_id = vpc_obj['default_security_group']['id']

        # update sg name
        sg_name = '{}-sg'.format(vpc_name)
        ibm_vpc_client.update_security_group(
            sg_id, security_group_patch={'name': sg_name})

        # add all other required rules configured by the specific backend
        for rule in self.sg_rules.keys():
            sg_rule_prototype = self._build_security_group_rule_prototype_model(
                rule)
            if sg_rule_prototype:
                res = ibm_vpc_client.create_security_group_rule(
                    sg_id, sg_rule_prototype).get_result()
        logger.info(f"Security group {sg_name} been updated with required rules")

    def _select_resource_group(self, auto=False):
        # find resource group

        @spinner
        def get_res_group_objects():
            return self.resource_service_client.list_resource_groups().get_result()['resources']

        res_group_objects = get_res_group_objects()
        if auto:
            return res_group_objects[0]['id']

        default = find_default(
            self.defaults, res_group_objects, id='resource_group_id')
        res_group_obj = get_option_from_list(
            "Select resource group", res_group_objects, default=default)
        return res_group_obj['id']

    def _select_zone(self, vpc_id, region, auto=False):
        # find availability zone
        @spinner
        def get_zones_and_subnets():
            zones_objects = self.ibm_vpc_client.list_region_zones(region).get_result()['zones']
            all_subnet_objects = self.ibm_vpc_client.list_subnets().get_result()['subnets']
            return zones_objects, all_subnet_objects

        zones_objects, all_subnet_objects = get_zones_and_subnets()
        
        if auto:
            return zones_objects[0]

        if vpc_id:
            # filter out zones that given vpc has no subnets in
            zones = [s_obj['zone']['name']
                        for s_obj in all_subnet_objects if s_obj['vpc']['id'] == vpc_id]
            zones_objects = [
                z for z in zones_objects if z['name'] in zones]
        logger.info(f"Zone chosen: {zones_objects[0]['name']}")
        return zones_objects[0] # returning a random zone the vpc has a subnet in.

    def _select_vpc(self, node_config, region):

        vpc_id, vpc_name, zone_obj, sg_id, resource_group = None, None, None, None, None
        ibm_vpc_client = self.ibm_vpc_client
       
        while True:

            @spinner
            def list_vpcs():
                return ibm_vpc_client.list_vpcs().get_result()['vpcs']
            @spinner
            def _create_vpc(vpc_name):
                return ibm_vpc_client.create_vpc(address_prefix_management='auto', classic_access=False,
                                                name=vpc_name, resource_group=resource_group).get_result()
                                                
            vpc_objects = list_vpcs()
            zone_obj = self._select_zone(vpc_id, region)
 
            resource_group_id = self._select_resource_group()
            resource_group = {'id': resource_group_id}

            # Create a unique VPC name
            vpc_name = append_random_suffix(base_name=self.vpc_name)
            vpc_obj = _create_vpc(vpc_name)
            vpc_id = vpc_obj['id']
            store_output({'vpc_id':vpc_id},self.base_config)

            logger.info(color_msg(f"Created VPC: {vpc_name} with id: {vpc_id}",color=Color.LIGHTGREEN))

            self._create_vpc_peripherals(ibm_vpc_client, vpc_obj, zone_obj, resource_group)
            break

        CACHE['resource_group_id'] = resource_group['id']
        
        return vpc_obj, zone_obj

    



# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
from vpc_img_inst.config_builder import ConfigBuilder, update_decorator, spinner
from typing import Any, Dict
from vpc_img_inst.utils import logger,color_msg,Color, store_output


class EndpointConfig(ConfigBuilder):
    
    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)
        
    @spinner
    def _get_regions_objects(self):
        return self.ibm_vpc_client.list_regions().get_result()['regions']
        
    @update_decorator
    def run(self) -> Dict[str, Any]:
        regions_objects = self._get_regions_objects()
        region_obj = next((r for r in regions_objects if r['name'] == self.base_config['region']), None)
        if not region_obj:
             logger.critical(color_msg(f"Region Chosen: {self.region} is invalid ",color=Color.RED))
             raise Exception("Invalid argument")
        self.ibm_vpc_client.set_service_url(region_obj['endpoint'] + '/v1')   # update global ibm_vpc_client to selected endpoint
        ConfigBuilder.region = region_obj['name']
        store_output({'endpoint':region_obj['endpoint']},self.base_config)  # to enable user to manually execute the clean-up module at a later time.  
        
        logger.info(f"Region Chosen: {region_obj['name']} ")
        return region_obj['endpoint']

    def update_config(self, endpoint):
        self.base_config['endpoint'] = endpoint
        self.base_config['region'] = ConfigBuilder.region






# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Dict
from vpc_img_inst.config_builder import ConfigBuilder, update_decorator
from vpc_img_inst.utils import password_dialog, color_msg, Color, verify_iam_api_key, store_output

class ApiKeyConfig(ConfigBuilder):

    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)
        self.defaults['api_key'] = self.base_config['iam_api_key'] if 'iam_api_key' in self.base_config \
            else None

    @update_decorator
    def run(self, api_key=None, compute_iam_endpoint=None) -> Dict[str, Any]:

        ConfigBuilder.compute_iam_endpoint = compute_iam_endpoint
        
        if not api_key:
            default = self.defaults.get('api_key')
            api_key = password_dialog("Please provide " + color_msg("IBM API KEY", color=Color.CYAN),
                                  default=default,
                                  validate=verify_iam_api_key)['answer']

        ConfigBuilder.iam_api_key = api_key
        store_output({'iam_api_key':api_key},self.base_config)
        return api_key, compute_iam_endpoint

    def update_config(self, iam_api_key, compute_iam_endpoint=None) -> Dict[str, Any]:
        self.base_config['iam_api_key'] =  iam_api_key
        
        if compute_iam_endpoint:
            self.base_config['iam_endpoint'] =  compute_iam_endpoint
            
        return self.base_config
    
    def verify(self, base_config):
        api_key = base_config['iam_api_key']
            
        ConfigBuilder.compute_iam_endpoint = self.base_config['iam_endpoint']
        
        verify_iam_api_key(None, api_key, iam_endpoint=ConfigBuilder.compute_iam_endpoint)
        ConfigBuilder.iam_api_key = api_key
            
        return base_config


# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from vpc_img_inst.config_builder import ConfigBuilder, spinner
from typing import Any, Dict
import paramiko
import time
import os
from vpc_img_inst.utils import color_msg, Color, logger, get_unique_file_name, DEFAULTS

class FeatureInstall(ConfigBuilder):
    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)
        self.installation_scripts = self.base_config['scripts_path']
        

    def run(self) -> Dict[str, Any]:

        @spinner
        def _run_remote_script(script):
            feature = script[script.find('_')+1:script.rfind('_')]
            self.inst_retries = DEFAULTS['script_inst_retries'] # reset retries for each script
            
            while self.inst_retries:  
                self.inst_retries -= 1          
                install_log = get_unique_file_name("installation_log", self.base_config['output_folder'])
                logger.info(color_msg(f"\nInstalling {feature} in newly created VSI.\n- See logs at {install_log}. Process might take a while.", color=Color.YELLOW))

                stdout = client.exec_command(f'chmod 777 {remote_destination}/{script_name}')[1] # returns the tuple (stdin,stdout,stderr)
                stdout = client.exec_command(f'{remote_destination}/{script_name}')[1]
                
                with open(install_log, "a") as f:
                    for line in stdout:
                        f.write(line)

                # Blocking call. returns when script completed/failed
                exit_status = stdout.channel.recv_exit_status()         
                if exit_status == 0:
                    logger.info(color_msg(f"""installation script "{script}" executed successfully.""",color=Color.LIGHTGREEN))
                    return True
                elif self.inst_retries:
                    time.sleep(2)
                    logger.error(color_msg(f"""Error executing "{script}". Retrying...""",color=Color.RED))

            raise Exception("Script installation failed. Terminating program.")

        @spinner
        def connect_to_ssh_port(key_obj):
            time.sleep(4)
            logger.info(f"user {self.base_config['auth']['ssh_user']} connecting to ssh port on host {self.base_config['node_config']['ip_address']} ")
            tries = 10
            msg = "Failed to connect to ssh port"
            while tries:
                try:
                    client.connect(self.base_config['node_config']['ip_address'], username=self.base_config['auth']['ssh_user'],pkey=key_obj)
                    return 
                except:
                    print(msg + ". Retrying..." if tries > 0 else msg)
                    tries -= 1
                    time.sleep(4)
            logger.critical(color_msg("\nFailed to connect to VSI via SSH. Terminating.\n", Color.RED))
            raise Exception("Failed to create to remote VM")

        for script in self.installation_scripts:

            file_to_execute_path = script
            script_name = file_to_execute_path.split(os.sep)[-1]
            remote_destination = "/tmp"

            # Connect to remote host
            key_obj = paramiko.RSAKey.from_private_key_file(self.base_config['auth']['ssh_private_key'])
            client = paramiko.SSHClient()
            client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            connect_to_ssh_port(key_obj)

            # Setup sftp connection and transmit script
            sftp = client.open_sftp()
            sftp.put(f'{file_to_execute_path}',f"{remote_destination}/{script_name}")
            sftp.close()
            _run_remote_script(script)
            client.close()

        return self.base_config



# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from vpc_img_inst.config_builder import ConfigBuilder
from typing import Any, Dict
from vpc_img_inst.utils import color_msg, Color, logger, store_output, append_random_suffix

class FloatingIpConfig(ConfigBuilder):
    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)
        self.base_config = base_config
    
    def run(self) -> Dict[str, Any]:
        fip_data = self.create_ip()
        self.base_config['node_config']['ip_address'] = fip_data['address']
        self.base_config['node_config']['ip_id'] = fip_data['id']
        store_output({'ip_id':fip_data['id']},self.base_config)
        store_output({'ip_address':fip_data['address']},self.base_config)
        return self.base_config

    def create_ip(self):
        ip_name = append_random_suffix(base_name="temp-fp")
        
        floating_ip_prototype = {}
        floating_ip_prototype["name"] = ip_name
        floating_ip_prototype["zone"] = {"name": self.base_config['zone_name']}
        floating_ip_prototype["resource_group"] = {
            "id": self.base_config['node_config']["resource_group_id"]
        }

        response = self.ibm_vpc_client.create_floating_ip(floating_ip_prototype)
        floating_ip_data = response.result
        logger.info(color_msg(f"Created floating ip: {floating_ip_data['address']} with id: {floating_ip_data['id']}", color=Color.LIGHTGREEN))
        
        self.attach_ip(floating_ip_data)
        return floating_ip_data
    
    def attach_ip(self, floating_ip_data):
        # Attach the ip to the VSI's network interface
        instance_data = self.ibm_vpc_client.get_instance(self.base_config['node_config']['vsi_id']).get_result()
        try:
            self.ibm_vpc_client.add_instance_network_interface_floating_ip(
                instance_data["id"], instance_data["network_interfaces"][0]["id"], floating_ip_data["id"])
        except Exception as e:
            logger.error("Failed to attach ip to VSI's network interface\nError: ",e)
            raise
        
        return floating_ip_data['address']







# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
from setuptools.command.install import install
import os 
import shutil
from pathlib import Path
PACKAGE_NAME = 'vpc-img-inst'
INSTALLATION_DIR_NAME = 'installation_scripts'
USER_SCRIPTS_FOLDER = str(Path.home())+os.sep+f".{PACKAGE_NAME}" 
PROJECT_ROOT = os.path.join(os.path.dirname(os.path.realpath(__file__)),'src','vpc_img_inst')

class CustomInstallCommand(install):
    """
    This class inherits from setuptools' install class to create the users scripts folder
    post installation ((pip) install) and copy the project's scripts to said folder. 
    """
    
    def run(self):
        install.run(self)
        
        if not os.path.exists(USER_SCRIPTS_FOLDER):
            print(f'Copying project scripts to {USER_SCRIPTS_FOLDER}')
            src = os.path.join(PROJECT_ROOT,INSTALLATION_DIR_NAME)
            dest = USER_SCRIPTS_FOLDER
            shutil.copytree(src, dest)



def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

find_packages(where='src',exclude=['logs'])

setup(
    name='vpc-img-inst',
    version='1.0.0b6',
    author =' Omer-J-Cohen',
    author_email = 'omer.cohen@ibm.com',
    description = 'vpc-img-inst is a lightweight script for quick-and-dirty generation of custom VSI images by installing features (software bundles) on base images.',
    long_description=read('README.md'),
    long_description_content_type = "text/markdown",
    url = 'https://github.com/IBM/vpc-img-inst',
    install_requires=[
        'click==8.0.4',
        'ibm_cloud_sdk_core==3.16.0',
        'ibm_platform_services==0.27.0',
        'ibm_vpc==0.12.0',
        'ibm_watson==6.1.0',
        'inquirer==2.10.1',
        'paramiko==2.11.0',
        'PyYAML==6.0',
        'setuptools==63.2.0'
    ],

    # Creates script named vpc-img-inst under ~/<python_folder>/bin/vpc-img-inst.
    # it is asked to pass parameters to builder() of vpc_img_inst.main. 
    entry_points={
    'console_scripts': ['vpc-img-inst=vpc_img_inst:main.builder']
    },
    # include otherwise undetected installation scripts and defaults.yaml  
    package_data={'': ['*.sh','*.yaml']},
    # inform setuptools to modify standard installation class. in this instance: setuptools.command.install 
    cmdclass={'install': CustomInstallCommand},
    python_requires = ">=3.6",
)




# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
from typing import Any, Dict
from pathlib import Path
from inquirer import errors
from vpc_img_inst.config_builder import ConfigBuilder, spinner
from vpc_img_inst.utils import Color, color_msg, get_unique_file_name,logger, append_random_suffix, store_output
from ibm_cloud_sdk_core import ApiException
DEFAULT_KEY_NAME = 'temp'
        
def generate_keypair():
    """Returns newly generated public ssh-key's contents and private key's path"""
    key_name= f"id_rsa.{DEFAULT_KEY_NAME}"
    home = str(Path.home())
    ssh_folder_path = f"{home}{os.sep}.ssh{os.sep}"
    unique_file_name = get_unique_file_name(key_name,ssh_folder_path)

    os.system(f'ssh-keygen -b 2048 -t rsa -f {unique_file_name} -q -N ""')

    logger.info(color_msg(f"Generated private key: {os.path.abspath(unique_file_name)}",color=Color.LIGHTGREEN))
    logger.info(color_msg(f"Generated public key: {os.path.abspath(unique_file_name)}.pub",color=Color.LIGHTGREEN))

    with open(f"{unique_file_name}.pub", 'r') as file:
        ssh_key_data = file.read()
    ssh_key_path = os.path.abspath(unique_file_name)
    return ssh_key_data, ssh_key_path

def get_ssh_key(ibm_vpc_client, name):
    """Returns ssh key matching specified name, stored in the VPC associated with the vpc_client"""
    for key in ibm_vpc_client.list_keys().result['keys']:
        if key['name'] == name:
            return key
                    
def register_ssh_key(ibm_vpc_client, config):
    """Returns the key's name on the VPC platform, it's public key's contents and the local path to it.
        Registers an either existing or newly generated ssh-key to a specific VPC. """
    resource_group_id = config['node_config']['resource_group_id']
    unique_key_name = append_random_suffix(base_name=DEFAULT_KEY_NAME)
    ssh_key_data, ssh_key_path = generate_keypair()

    response = None
    try:    # regardless of the above, try registering an ssh-key 
        response = ibm_vpc_client.create_key(public_key=ssh_key_data, name=unique_key_name, resource_group={
                                        "id": resource_group_id}, type='rsa')
    except ApiException as e:
        print(e)
        if "Key with fingerprint already exists" in e.message:
            logger.info(color_msg("Can't register an SSH key with the same fingerprint",Color.RED))
        raise # can't continue the configuration process without a valid ssh key     
        
    result = response.get_result()
    logger.info(color_msg(f"Registered SSH key: {unique_key_name} with id: {result['id']} to resource group\n",Color.LIGHTGREEN))
    return result['name'], result['id'], ssh_key_path

class SshKeyConfig(ConfigBuilder):

    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)
        self.base_config = base_config

    def _validate_keypair(self, answers, current):
        if not current or not os.path.exists(os.path.abspath(os.path.expanduser(current))):
            raise errors.ValidationError(
            '', reason=f"File {current} doesn't exist")

        public_res = self.ibm_vpc_client.get_key(self.ssh_key_id).get_result()['public_key'].split(' ')[1]
        private_res = subprocess.getoutput([f"ssh-keygen -y -f {current} | cut -d' ' -f 2"])

        if not public_res == private_res:
            raise errors.ValidationError(
            '', reason=f"Private ssh key {current} and public key {self.ssh_key_name} are not a pair")

        return public_res == private_res
    
    @spinner
    def get_ssh_key_names(self):
        return [key['name'] for key in self.ibm_vpc_client.list_keys().get_result()['keys']]

    def run(self) -> Dict[str, Any]:
        ssh_key_name, ssh_key_id, ssh_key_path = register_ssh_key(self.ibm_vpc_client, self.base_config)
        self.ssh_key_id = ssh_key_id
        self.ssh_key_name = ssh_key_name

        self.base_config['node_config']['key_id'] = ssh_key_id
        self.base_config['auth']['ssh_user'] = 'root' # user is hardcoded to root 
        self.base_config['auth']['ssh_private_key'] = ssh_key_path
        store_output({'ssh_private_key':ssh_key_path},self.base_config)
        store_output({'key_id':ssh_key_id},self.base_config)
        return self.base_config


# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Dict
from vpc_img_inst.config_builder import ConfigBuilder, update_decorator, spinner
from vpc_img_inst.utils import logger, color_msg, Color, CACHE



class ImageConfig(ConfigBuilder):
    
    def __init__(self, base_config: Dict[str, Any]) -> None:
        super().__init__(base_config)

    @spinner
    def get_image_objects(self):
        images = []
        res = self.ibm_vpc_client.list_images().get_result()
        images.extend(res['images'])

        while res.get('next',None):
            link_to_next = res['next']['href'].split('start=')[1].split('&limit')[0]
            res = self.ibm_vpc_client.list_images(start=link_to_next).get_result()
            images.extend(res['images'])
        return images

    def run(self) -> Dict[str, Any]:
        image_objects = self.get_image_objects()
        # picks the first image with AMD architecture (opposed to s390x) that contains prefix matching user's input (-im flag).   
        image = next((img for img in image_objects if img['name'].startswith(self.base_config['user_image_name']) \
            and img['operating_system']['architecture'].startswith('amd')), None)
        
        if not image:
            logger.critical(color_msg(f"Base image chosen: {self.base_config['user_image_name']} doesn't exist. ",color=Color.RED))
            raise Exception
            
        self.base_config['node_config']['image_id'] = image['id']
        self.base_config['node_config']['image_name'] = image['name']
        self.base_config['node_config']['boot_volume_capacity'] = self.base_config['node_config'].get("boot_volume_capacity", 100)

        logger.info(f"image Chosen: {image['name']}")
        return self.base_config





#!/usr/bin/python3
# -*- coding: utf-8 -*-
import re
import sys
from vpc_img_inst.main import builder

def entry():
    sys.argv[0] = re.sub(r'(-script\.pyw|\.exe)?$', '', sys.argv[0])
    sys.exit(builder())









# (C) Copyright IBM Corp. 2022.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import tempfile
from pathlib import Path
DIR_PATH = os.path.dirname(os.path.realpath(__file__))  # absolute path to project's root folder.
USER_SCRIPTS_FOLDER = str(Path.home())+os.sep+".vpc-img-inst" 

DEFAULTS = {'base_image_name':'ibm-ubuntu-20-04',  
            'region':'eu-de',
             'installation_type':"ubuntu",
             "feature":"cuda", 
             "image_create_retries":3, 
             "script_inst_retries":5,
             "input_file":f'{DIR_PATH}{os.sep}defaults.yaml',
             "output_folder":tempfile.mkdtemp()}


from vpc_img_inst.config_modules.apikey_config import ApiKeyConfig
from vpc_img_inst.config_modules.endpoint_config import EndpointConfig
from vpc_img_inst.config_modules.ip_config import FloatingIpConfig
from vpc_img_inst.config_modules.ssh_config import SshKeyConfig
from vpc_img_inst.config_modules.vpc_config import VPCConfig
from vpc_img_inst.config_modules.vsi_config import VSIConfig
from vpc_img_inst.config_modules.image_config import ImageConfig
from vpc_img_inst.config_modules.install_feature import FeatureInstall
from vpc_img_inst.config_modules.image_create import ImageCreate
from vpc_img_inst.config_modules.delete_resources import DeleteResources


MODULES = [ApiKeyConfig, EndpointConfig, VPCConfig, ImageConfig,
            SshKeyConfig, VSIConfig, FloatingIpConfig, FeatureInstall, ImageCreate, DeleteResources]